/*
 * DocDoku, Professional Open Source
 * Copyright 2006 - 2013 DocDoku SARL
 *
 * This file is part of DocDokuPLM.
 *
 * DocDokuPLM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DocDokuPLM is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU Affero General Public License for more details.  
 *  
 * You should have received a copy of the GNU Affero General Public License  
 * along with DocDokuPLM.  If not, see <http://www.gnu.org/licenses/>.  
 */

package com.docdoku.core.meta;

import java.util.Date;
import javax.persistence.Entity;
import javax.persistence.Table;

/**
 * Defines a date type custom attribute of a document.
 * 
 * @author Florent Garin
 * @version 1.0, 02/06/08
 * @since   V1.0
 */
@Table(name="INSTANCEDATEATTRIBUTE")
@Entity
public class InstanceDateAttribute extends InstanceAttribute{

    
    @javax.persistence.Temporal(javax.persistence.TemporalType.TIMESTAMP)
    private Date dateValue;
    
    public InstanceDateAttribute() {
    }
    
    public InstanceDateAttribute(String pName, Date pValue) {
        super(pName);
        setDateValue(pValue);
    }

    @Override
    public Date getValue() {
        return dateValue;
    }

    public Date getDateValue() {
        return dateValue;
    }

    public void setDateValue(Date dateValue) {
        this.dateValue = dateValue;
    }
    
    
    /**
     * perform a deep clone operation
     */
    @Override
    public InstanceDateAttribute clone() {
        InstanceDateAttribute clone = null;
        clone = (InstanceDateAttribute) super.clone();
        
        if(dateValue!=null)
            clone.dateValue = (Date) dateValue.clone();
        return clone;
    }

    @Override
    public boolean setValue(Object pValue) {
        if(pValue instanceof Date){
            dateValue=(Date)pValue;
            return true;
        }else
            return false;
    }

}
